"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const sinon = require("sinon");
const chai = require("chai");
chai.use(require("chai-as-promised"));
const fs = require("fs-extra");
const path = require("path");
const proxyquire = require("proxyquire");
const Token_1 = require("../shared/Token");
describe('SyncProxyService', () => {
    let service;
    let discovery;
    let server;
    let privateKey;
    let publicKey;
    let httpProxy;
    let proxy;
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        discovery = {
            find: sinon.stub(),
        };
        privateKey = yield fs.readFile(path.resolve(__dirname, "../../fixtures/keys/auth.key"), "utf-8"),
            publicKey = yield fs.readFile(path.resolve(__dirname, "../../fixtures/keys/auth.pub"), "utf-8"),
            server = sinon.stub({
                publicKey,
                discovery,
            });
        proxy = {
            on: sinon.stub(),
            ws: sinon.stub(),
        };
        httpProxy = {
            createProxy: sinon.stub().returns(proxy),
        };
        const { SyncProxyService } = proxyquire("./SyncProxyService", { 'http-proxy': httpProxy });
        service = new SyncProxyService();
        yield service.start(server);
    }));
    function runHandler(service, req) {
        return __awaiter(this, void 0, void 0, function* () {
            const socket = { destroy: sinon.stub() };
            const head = sinon.stub();
            yield service.websocketHandler(req, socket, head);
        });
    }
    function createAuthorizationHeader(token, version = 1) {
        return `Realm-Access-Token version=${version} token="${token.sign(privateKey)}"`;
    }
    describe("websocketHandler", () => {
        const validToken = new Token_1.AccessToken({
            access: ["download"],
            identity: "some-user",
            appId: "io.realm.Test",
            path: "/some-uid/myrealm",
            syncLabel: "default",
        });
        const noSyncLabelToken = new Token_1.AccessToken({
            access: ["download"],
            identity: "some-user",
            appId: "io.realm.Test",
            path: "/some-uid/myrealm"
        });
        describe("without an authorization header", () => {
            it("should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                yield chai_1.assert.isRejected(runHandler(service, {
                    headers: {},
                }), "Authorization header was not provided");
            }));
        });
        describe("with an invalid access token header", () => {
            it("should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                yield chai_1.assert.isRejected(runHandler(service, {
                    headers: {
                        'authorization': "gibberish",
                    },
                }), "Authorization header is not in a valid format");
            }));
        });
        describe("with a valid access token header", () => {
            it("should open a proxy connection", () => __awaiter(this, void 0, void 0, function* () {
                discovery.find.withArgs("sync", ["master", "default"]).returns({});
                yield chai_1.assert.isFulfilled(runHandler(service, {
                    headers: {
                        'authorization': createAuthorizationHeader(validToken),
                    },
                    params: {
                        path: "/some-uid/myrealm",
                    }
                }));
                sinon.assert.calledOnce(discovery.find);
                sinon.assert.calledOnce(httpProxy.createProxy);
                sinon.assert.calledTwice(proxy.on);
                sinon.assert.calledOnce(proxy.ws);
            }));
            describe("without a syncLabel", () => {
                it("should lookup the location", () => __awaiter(this, void 0, void 0, function* () {
                    discovery.find.withArgs("realms").returns({
                        address: "127.0.0.1",
                        port: 80
                    });
                    yield chai_1.assert.isRejected(runHandler(service, {
                        headers: {
                            'authorization': createAuthorizationHeader(noSyncLabelToken),
                        },
                        params: {
                            path: "/some-uid/myrealm",
                        }
                    }), "AccessToken is not valid: sync label was not found in token");
                    sinon.assert.calledOnce(discovery.find);
                }));
            });
        });
    });
});
//# sourceMappingURL=SyncProxyService.spec.js.map